#!/usr/bin/env python3

import gi, sys, os, mutagen, threading
# from pprint import pprint as print

gi.require_version('Gtk', '3.0')

from gi.repository import Gtk, Gdk, GLib, GdkPixbuf, GObject

from utils import blur_image, pic_to_pixbuf, base64_to_pixbuf

from ui.image import ScaleImage
from ui.slider import Slider
from ui.image_button import ImageButton
from ui.text import TextBox
from ui.ctrl_box import CtrlBox
from ui.timebar import Timebar
from ui.option_menu import OptionMenu


# 定义一个修饰器, 用于将当前方法转到主线程中运行 (子线程中调用方法时)
def idle(func):
  def wrapper(*args):
    GObject.idle_add(func, *args)
  return wrapper


class SonistWindow(Gtk.Window):
  app = None
  mpd = None
  stat = {}

  def __init__(self, app):
    Gtk.Window.__init__(self)

    self.app = app
    self.mpd = app.mpd

    self.connect("destroy", self.quited)

    self.mpd.connect('offline', lambda o: self.reset_player())
    self.mpd.connect('online', lambda o: self.sync_state(None, None, True))
    self.mpd.connect('state_changed', lambda o, stat: self.update_play_stat(stat == 'play'))
    self.mpd.connect('song_changed', lambda o, stat, song: self.sync_state(stat, song, False))
    self.mpd.connect('playing', lambda o, stat, song: self.update_playtime(stat))

    self.set_name('SonistWindow')
    self.set_default_size(320, 384)
    self.set_resizable(False)
    self.set_wmclass('Sonist', 'Sonist')

    self.set_opacity(0.9)

    # self.set_keep_above(True)

    # album_img = './usr/share/sonist/album.png'
    album_img = './usr/share/sonist/avatar.jpg'

    self.set_background_image(album_img)

    layout = Gtk.Layout()

    # 菜单按钮
    menu_btn = ImageButton('./usr/share/sonist/menu.png')
    popup_menu = OptionMenu()
    menu_btn.connect('clicked', lambda w: popup_menu.show(w))
    layout.put(menu_btn, 276, 6)
    
    # 唱片
    disk = ScaleImage('./usr/share/sonist/disk.png')
    handler = ScaleImage('./usr/share/sonist/handler.png')
    album = ScaleImage(album_img)

    disk.resize(192, 192)
    album.resize(128, 128).set_radius(64)

    handler.resize(48, 96)
    self.handler = handler
    self.album = album

    box = Gtk.Fixed()
    box.put(disk, 16, 16)
    box.put(album, 48, 48)
    box.put(handler, 0, 16)

    layout.put(box, 48, 16)


    # title

    # self.title_box = TextBox(256, 20)
    self.title_box = Gtk.Label()
    self.title_box.set_name('text')
    self.title_box.set_text('mpd loading...')

    layout.put(self.title_box, 27, 244)


    # 播放进度
    self.timebar = Timebar()
    self.timebar.connect('seeked', lambda a,v: self.mpd.seekcur(v))
    layout.put(self.timebar, 24, 270)


    # 控制条
    self.ctrl_box = CtrlBox()
    self.ctrl_box.connect('clicked', self.ctrl_clicked)


    layout.put(self.ctrl_box, 48, 312)

    self.add(layout)

 
    


  def get_mpd_stat(self):
    try:
      self.stat = self.mpd.status()
    except:
      self.stat = {}
    return self.stat


  def set_background_image(self, filepath):
    if type(filepath) == str:
      pixbuf = GdkPixbuf.Pixbuf.new_from_file(filepath)
    else:
      pixbuf = filepath
    
    pixbuf = blur_image(pixbuf)
    pixbuf.savev(f"/tmp/sonist_album_cache", 'png', [], [])

    css = f"""
    #SonistWindow {{
      background-image: url('/tmp/sonist_album_cache');
      background-size: 100% 100%;
      background-position: center;
    }}
    #text {{
      color: #f2f5fc;
    }}
    """
    # 加载CSS样式
    css_provider = Gtk.CssProvider()
    css_provider.load_from_data(css.encode('UTF-8'))
    context = Gtk.StyleContext()
    screen = Gdk.Screen.get_default()
    context.add_provider_for_screen(screen, css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)


  def ctrl_clicked(self, box, btn):
    match(btn):
      case 'play_btn':
        self.toggle_play()

      case 'mode_btn':
        # repeat all
        if self.ctrl_box.curr_mode == 0:
          self.mpd.repeat(1)
          self.mpd.random(0)
          self.mpd.single(0)
        # random
        elif self.ctrl_box.curr_mode == 1:
          self.mpd.repeat(0)
          self.mpd.random(1)
          self.mpd.single(0)
        # single
        else:
          self.mpd.repeat(0)
          self.mpd.random(0)
          self.mpd.single(1)
      
      case 'prev_btn':
        self.prev_song()

      case 'next_btn':
        self.next_song()

      case 'vol_btn':
        self.toggle_play()



  def toggle_play(self):
    try:
      if self.stat.get('state') == 'stop':
        self.mpd.play()
      else:
        self.mpd.pause()
    except:
      pass

    self.update_play_stat(self.stat.get('state') == 'play')


  def prev_song(self): 
    try:
      self.mpd.previous()
    except:
      pass

  def next_song(self):
    try:
      self.mpd.next()
    except:
      pass

  @idle
  def update_play_stat(self, played = True):
    if not self.mpd.connected:
      return 
    
    if played:
      self.handler.reset('./usr/share/sonist/handler_a.png')
    else:
      self.handler.reset('./usr/share/sonist/handler.png')

    # 切换播放按钮状态
    self.ctrl_box.toggle_play_btn(played)

  @idle
  def update_playtime(self, stat = {}):
    times = stat['time'].split(':')
    self.timebar.update_time(int(times[0]), int(times[1]))
    
  @idle
  def sync_state(self, stat = None, song = None, first = False):
    self.ctrl_box.disabled = False
    print(threading.active_count())
    print(threading.current_thread())
    
    self.stat = stat or self.get_mpd_stat()

    played = self.stat.get('state')

    if first:
      self.update_play_stat(played == 'play')

      if self.stat.get('single') == '1':
        self.ctrl_box.toggle_mode_btn(mode = 'single')
      elif self.stat.get('random') == '1':
        self.ctrl_box.toggle_mode_btn(mode = 'random')


    if played != 'stop':
      song = song or self.mpd.currentsong()
      # 更新歌曲信息
      self.title_box.set_text(f"{song.get('artist')} - {song.get('title')}")
      self.update_playtime(self.stat)

      filepath = f"./album/{song['title']}.png"
      songpath = f"{self.app.music_dir}/{song['file']}"

      if os.path.isfile(filepath):
        self.update_album(filepath)
      else:
    
        id3 = mutagen.File(songpath)

        try:
          if id3.tags.get('APIC:'):
            pic = id3.tags['APIC:']
          elif len(id3.pictures) > 0:
            pic = id3.pictures[0]

          if pic is not None:
            album = pic_to_pixbuf(pic)
            self.update_album(album)
        except:
          pass
  
  @idle
  def reset_player(self):
    self.ctrl_box.disabled = True
    self.title_box.set_text('mpd is offline...')
    self.timebar.update_time()
    self.update_album('./usr/share/sonist/avatar.jpg')

  @idle
  def update_album(self, filepath):
    self.set_background_image(filepath)
    self.album.reset(filepath).set_radius(64)


  def quited(self, win):
    self.app.remove_window(self)

